#!/usr/bin/python
import hal, time
h = hal.component("A2GantryController")

h.newpin("state-in"          , hal.HAL_FLOAT      , hal.HAL_IN)
h.newpin("estop-in"          , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("machine_is_on-in"  , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("home_button-in"    , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("granite_is_homingX" , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("granite_is_homingY" , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("ServoFaultX"      , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("ServoFaultY1"     , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("ServoFaultY2"     , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("ServoFaultZ"      , hal.HAL_BIT        , hal.HAL_IN)

h.newpin("EMC_MODE_Is_MDI"   , hal.HAL_BIT        , hal.HAL_IN)
h.newpin("EMC_MODE_Is_Manual", hal.HAL_BIT        , hal.HAL_IN)
h.newpin("EMC_MODE_Is_Teleop", hal.HAL_BIT        , hal.HAL_IN)
h.newpin("EMC_MODE_Is_Joint" , hal.HAL_BIT        , hal.HAL_IN)

h.newpin("EMC_Is_Homed"      , hal.HAL_BIT        , hal.HAL_IN)

h.newpin("state-out"         , hal.HAL_FLOAT      , hal.HAL_OUT)
h.newpin("state-change"      , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("enable-granites"   , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("home-granites"     , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("reset-encoders"    , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("machine-off"       , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("machine-on"        , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("home_emc"           , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("EMC_MODE_Set_MDI"   , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("EMC_MODE_Set_Manual", hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("EMC_MODE_Set_Teleop", hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("EMC_MODE_Set_Joint" , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("home_in_progress"   , hal.HAL_BIT        , hal.HAL_OUT)
h.newpin("clear_UI_errors"    , hal.HAL_BIT        , hal.HAL_OUT)

h.newpin("A2GC_Debug"         , hal.HAL_FLOAT      , hal.HAL_OUT)

try:
	states = {"EStop":0.0, "ReadyOff":1.0, "ReadyOn":2.0, "HomingEx":3.0}
	state = states["EStop"]
	
	h["state-out"] = state
	h["state-change"] = 0
	h["A2GC_Debug"] = 0
	
	h.ready()

	while 1:
		time.sleep(0.01)

		h["home-granites"] = 0
		h["reset-encoders"] = 0
		h["machine-off"] = 0
		h["machine-on"] = 0
		h["home_emc"] = 0
		h["EMC_MODE_Set_MDI"] = 0
		h["EMC_MODE_Set_Manual"] = 0
		h["EMC_MODE_Set_Teleop"] = 0
		h["EMC_MODE_Set_Joint"] = 0
		h["home_in_progress"] = 0
		h["clear_UI_errors"] = 0
		
		instate = h['state-in']

		if (   (h["ServoFaultX"] or h["ServoFaultY1"] or h["ServoFaultY2"] ) and ( h["machine_is_on-in"] ) ) :
			h["enable-granites"] = 0
			h["machine-off"] = 1
			while (h["machine_is_on-in"]):
				if (h['estop-in']) : break # Sanity check				
			h["machine-off"] = 0
			if ( instate == states["HomingEx"] ):
				instate = states["ReadyOff"]
				h["home_in_progress"] = 0

		if (h['estop-in']): # EStop always takes presidence
			state=states["EStop"]
			h["enable-granites"] = 0
			h["A2GC_Debug"] = -1

		elif (instate == states["EStop"]):
			h["enable-granites"] = 0
			h["A2GC_Debug"] = -2
			if (not h['estop-in']):
				state = states["ReadyOff"]
			else:
				state = instate
			
		elif (instate == states["ReadyOff"]):
			h["enable-granites"] = 0
			h["A2GC_Debug"] = -3

			if (h['machine_is_on-in']):
				state = states["ReadyOn"]
			elif (h["home_button-in"]):
				state = states["HomingEx"]
			else:
				state = instate
			
		elif (instate == states["ReadyOn"]):
			h["enable-granites"] = 1
			h["A2GC_Debug"] = -4
	
			if   (not h["machine_is_on-in"]):
				state = states["ReadyOff"]
			elif (h["home_button-in"] and h["EMC_MODE_Is_Manual"]):
				state = states["HomingEx"]
			else:
				state = instate	
			
		elif (instate == states["HomingEx"]):
			h["home_in_progress"] = 1
			h["enable-granites"] = 1
			h["A2GC_Debug"] = 1

			# always change to ReadyOn state after homing
			AfterHomeChangeTo = states["ReadyOn"]
			time.sleep(1) # let granites come out of disable mode

			h["A2GC_Debug"] = 999

			# turn the machine on
			while (not h["machine_is_on-in"]):
				h["machine-on"] = 1
				time.sleep(0.01)
				h["machine-on"] = 0
				if (h["estop-in"]) : break # Sanity check
			h["machine-on"] = 0
			if (h["estop-in"]) : continue # Sanity check
			h["A2GC_Debug"] = 60

			# Start EMC Homing procedure.  This pass is only for the Z axis, to make it move up and out of the way
			# A second homing of Z will be done after the granites have homed, so that EMC will know Z is in a good 
			# home position even after the machine is turned off and then on
			h["home_emc"] = 1
			time.sleep(1)
			while (not h["EMC_Is_Homed"]):
				if (h["estop-in"]) : break # Sanity check
			h["home_emc"] = 0
			if (h["estop-in"]) : continue # Sanity check
			time.sleep(1)

			h["A2GC_Debug"] = 110

			# Turn off the EMC machine so it doesn't try to monitor the drive motion
			# Turn ON the granites, so they will move and home
			h["machine-off"] = 1
			while (h["machine_is_on-in"]):
				if (h['estop-in']) : break # Sanity check				
			h["machine-off"] = 0
			if (h['estop-in']) : continue # Sanity check
			h["A2GC_Debug"] = 2
			
			# Start homing the granites - FALLING Edge of this signal triggers homing
			h["home-granites"] = 1
			time.sleep(0.1)
			h["home-granites"] = 0
			time.sleep(0.1)
			while ( (not h["granite_is_homingX"]) or (not h["granite_is_homingY"]) ):
				h["home-granites"] = 1
				time.sleep(0.1)
				h["home-granites"] = 0
				time.sleep(0.1)
				if (h["estop-in"]) : break # Sanity check
			if (h["estop-in"]) : continue # Sanity check
			h["A2GC_Debug"] = 3
			
			# wait for the granites to finish homing
			while (h["granite_is_homingX"] or h["granite_is_homingY"]):
				if (h["estop-in"]) : break # Sanity check
			if (h["estop-in"]) : continue # Sanity check
			h["A2GC_Debug"] = 4
			
			# reset encoders to 0, this is our new home position for the granite axes
			time.sleep(0.1)
			h["reset-encoders"] = 1
			time.sleep(0.1)
			h["reset-encoders"] = 0
			if (h["estop-in"]) : continue # Sanity check
			h["A2GC_Debug"] = 5
			
			# turn the machine back on
			while (not h["machine_is_on-in"]):
				h["machine-on"] = 1
				time.sleep(0.01)
				h["machine-on"] = 0
				if (h["estop-in"]) : break # Sanity check
			h["machine-on"] = 0
			if (h["estop-in"]) : continue # Sanity check
			h["A2GC_Debug"] = 6

			# Start EMC Homing procedure again to home Z for real, and mark X and Y as now homed
			h["home_emc"] = 1
			time.sleep(1)
			while (not h["EMC_Is_Homed"]):
				if (h["estop-in"]) : break # Sanity check
			h["home_emc"] = 0
			if (h["estop-in"]) : continue # Sanity check
			time.sleep(1)

			# reset encoders to 0, this is our new home position for the granite axes
			h["reset-encoders"] = 1
			time.sleep(0.1)
			h["reset-encoders"] = 0
			if (h["estop-in"]) : continue # Sanity check
			h["A2GC_Debug"] = 15

			# reset any errors regarding soft limits
			h["clear_UI_errors"] = 1
			time.sleep(0.1)
			h["clear_UI_errors"] = 0

			h["home_in_progress"] = 0
			state=AfterHomeChangeTo
			h["A2GC_Debug"] = 16
		
		state_changed = (instate != state)

		h['state-change'] = state_changed
		h['state-out'] = state
		
except KeyboardInterrupt:
	h["enable-granites"] = 0
	raise SystemExit
	