#!/usr/bin/env python

import linuxcnc
import os
import hal
import hal_glib
import Tkinter
import time
from gladevcp.persistence import IniFile,widget_defaults,set_debug,select_widgets

class HandlerClass:

    def __init__(self, halcomp,builder,useropts):
        print "lathepanel-settings.py: START"
        self.halcomp = halcomp
        self.builder = builder
        self.useropts = useropts
        self.defaults = {
			# the following names will be saved/restored as method attributes
			# the save/restore mechanism is strongly typed - the variables type will be derived from the type of the
			# initialization value. Currently supported types are: int, float, bool, string
			#IniFile.vars : { 'var1' : 1.11, 'var2' : 1.22, 'var3' : 0.333 },

			# Retains all HAL output widgets' state:
			IniFile.widgets: widget_defaults(select_widgets(self.builder.get_objects(), hal_only=True,output_only = True)),
			}

        self.ini_filename = __name__ + '.ini'
        self.ini = IniFile(self.ini_filename,self.defaults,self.builder)
        self.ini.restore_state(self)

        self.ReloadAction = self.builder.get_object('MDI_Reload_File')
        
        self.comm = linuxcnc.command()
        self.stat = linuxcnc.stat()

        print "lathepanel-settings.py: DONE"
		
    def vbox1_destroy_cb(self,obj,data=None):
        self.ini.save_state(self)

    def poll(self):
        self.stat.poll()

    def motion_mode(self):
        self.poll()
        return self.stat.motion_mode

    def mdi_exec(self,cmd):
        c = self.comm
        s = self.stat
        
        self.poll()
        
        if not s.estop and s.enabled and s.homed and (s.interp_state == linuxcnc.INTERP_IDLE):
            c.mode(linuxcnc.MODE_MDI)
            while c.wait_complete() == -1:
                pass

            c.mdi(cmd)
            time.sleep(0.2)
            while c.wait_complete() == -1:
                pass

    def mode_change(self,mode):
        c = self.comm
        s = self.stat
        
        self.poll()
        
        while (s.interp_state != linuxcnc.INTERP_IDLE):
            time.sleep(0.1)
            s.poll()
        
        time.sleep(0.2)
        c.mode(mode)
        while c.wait_complete() == -1:
            pass
         
    def on_gomachinezero_btn_pressed(self,obj,data=None):
        start_mode = self.motion_mode()

        self.mdi_exec("G53 G0 X0")
        self.mdi_exec("G53 G0 Z0")
        
        self.mode_change(start_mode)

    def on_known_state_btn_pressed(self,obj,data=None):
        start_mode = self.motion_mode()

        self.mdi_exec("M6 T0")
        self.mdi_exec("G10 L2 P0 X0Z0")
        self.mdi_exec("G8 G18 G20 G40 G49 G54 G90 G92.1 G94 G97")

        self.mode_change(start_mode)
                   
    def on_toolchange_btn_pressed(self,obj,data=None):
        start_mode = self.motion_mode()
                
        try:
            toolNum = {
                "T1": 1,
                "T2": 2,
                "T3": 3,
                "T4": 4,
                "T5": 5,
                "T6": 6,
                "T7": 7,
                "T8": 8,
            }[obj.get_label()]
        except:
            print "Toolchange: tool number unknown"
            return

        print "Toolchange: " + str(toolNum)
                    
        self.mdi_exec("G53 G0 X0")
        self.mdi_exec("G53 G0 Z0")
        self.mdi_exec("M6 T" + str(toolNum) + " G43")

        self.mode_change(start_mode)

def get_handlers(halcomp,builder,useropts):
    return [HandlerClass(halcomp,builder,useropts)]
	
